local UpdateInt = 30 -- Seconds between updates
local CombatTime = 0
local GetTime = GetTime
local RequestLFDPlayerLockInfo = RequestLFDPlayerLockInfo
local GetLFGRoleShortageRewards = GetLFGRoleShortageRewards
local LFG_ROLE_NUM_SHORTAGE_TYPES = LFG_ROLE_NUM_SHORTAGE_TYPES
local InCombatLockdown = InCombatLockdown
local format = format
local print = print
local pairs = pairs

local Roles = {"TANK", "HEALER", "DAMAGER"}
local RolesName = {TANK, HEALER, DAMAGER}

local BlankTex = "Interface\\AddOns\\Militant_CallToArms\\Blank.tga"
local BarTex = "Interface\\AddOns\\Militant_CallToArms\\Ferous.tga"
local Font = "Interface\\Addons\\Militant_CallToArms\\PTSans.ttf"

-- /run for i = 1, GetNumRandomDungeons() do print(GetLFGRandomDungeonInfo(i)) end

local Options = {}
Options.PrintChat = true -- Print messages into chat
Options.AnnounceEndings = false -- Announce when a queue ended

local CallToArms = CreateFrame("Frame", "CallToArmsGlobal", UIParent)
CallToArms:SetSize(130, 18)
CallToArms:SetPoint("LEFT", UIParent, 8, 0)
CallToArms:SetMovable(true)
CallToArms:EnableMouse(true)
CallToArms:SetUserPlaced(true)
CallToArms:RegisterForDrag("LeftButton")
CallToArms:SetScript("OnDragStart", CallToArms.StartMoving)
CallToArms:SetScript("OnDragStop", CallToArms.StopMovingOrSizing)
CallToArms.Ready = false
CallToArms.Ela = 0
CallToArms.Headers = {}

CallToArms.Backdrop = CallToArms:CreateTexture(nil, "BORDER")
CallToArms.Backdrop:SetPoint("TOPLEFT", CallToArms, -1, 1)
CallToArms.Backdrop:SetPoint("BOTTOMRIGHT", CallToArms, 1, -1)
CallToArms.Backdrop:SetTexture(BlankTex)
CallToArms.Backdrop:SetVertexColor(0, 0, 0)

CallToArms.Texture = CallToArms:CreateTexture(nil, "ARTWORK")
CallToArms.Texture:SetPoint("TOPLEFT", CallToArms, 0, 0)
CallToArms.Texture:SetPoint("BOTTOMRIGHT", CallToArms, 0, 0)
CallToArms.Texture:SetTexture(BarTex)
CallToArms.Texture:SetVertexColor(0.2, 0.2, 0.2)

CallToArms.Text = CallToArms:CreateFontString(nil, "OVERLAY")
CallToArms.Text:SetPoint("LEFT", CallToArms, 3, -0.5)
CallToArms.Text:SetFont(Font, 12)
CallToArms.Text:SetJustifyH("LEFT")
CallToArms.Text:SetShadowColor(0, 0, 0)
CallToArms.Text:SetShadowOffset(1.25, -1.25)
CallToArms.Text:SetText(BATTLEGROUND_HOLIDAY)

CallToArms.CloseButton = CreateFrame("Frame", nil, CallToArms)
CallToArms.CloseButton:SetPoint("TOPRIGHT", CallToArms, 0, 0)
CallToArms.CloseButton:SetSize(18, 18)
CallToArms.CloseButton:SetScript("OnEnter", function(self) self.Label:SetTextColor(1, 0, 0) end)
CallToArms.CloseButton:SetScript("OnLeave", function(self) self.Label:SetTextColor(1, 1, 1) end)
CallToArms.CloseButton:SetScript("OnMouseUp", function() CallToArms:Hide() end)

CallToArms.CloseButton.Label = CallToArms.CloseButton:CreateFontString(nil, "OVERLAY")
CallToArms.CloseButton.Label:SetPoint("CENTER", CallToArms.CloseButton, 0.5, -0.5)
CallToArms.CloseButton.Label:SetFont(Font, 18)
CallToArms.CloseButton.Label:SetText("×")
CallToArms.CloseButton.Label:SetShadowColor(0, 0, 0)
CallToArms.CloseButton.Label:SetShadowOffset(1.25, -1.25)

local OnUpdate = function(self, ela)
	self.Ela = self.Ela + ela
	
	if (self.Ela > UpdateInt) then
		RequestLFDPlayerLockInfo()
		self.Ela = 0
	end
end

function CallToArms:PLAYER_REGEN_ENABLED()
	self.Ela = self.Ela + (GetTime() - CombatTime)
	
	self:SetScript("OnUpdate", OnUpdate)
end

function CallToArms:PLAYER_REGEN_DISABLED()
	self:SetScript("OnUpdate", nil)
	CombatTime = GetTime()
end

function CallToArms:PLAYER_ENTERING_WORLD()
	if (UnitLevel("player") ~= 110) then -- We're only interested in max level information here.
		self:UnregisterAllEvents()
		self:Hide()
		
		return
	end
	
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function CallToArms:LFG_UPDATE_RANDOM_INFO()
	if (not self.Ready) then
		return
	end
	
	for Name, Header in pairs(self.Headers) do
		Header:Update()
	end
	
	if (not self:GetScript("OnUpdate") and (not InCombatLockdown())) then
		self:SetScript("OnUpdate", OnUpdate)
	end
end

function CallToArms:VARIABLES_LOADED()
	if (not Militant_CallToArms) then
		Militant_CallToArms = {}
	end

	for option, value in pairs(Militant_CallToArms) do
		Options[option] = value
	end
	
	self.Ready = true
end

function CallToArms:SortHeaders()
	local LastHeader = self
	
	for k, v in pairs(self.Headers) do
		if v:IsShown() then
			if (LastHeader == self) then
				v:SetPoint("TOPLEFT", LastHeader, "BOTTOMLEFT", 0, -1)
			else
				v:SetPoint("TOPLEFT", LastHeader.LastShown, "BOTTOMLEFT", 0, -1)
			end
			
			LastHeader = v
		end
	end
end

local OnEvent = function(self, event, ...)
	if self[event] then
		self[event](self, ...)
	end
end

CallToArms:RegisterEvent("PLAYER_ENTERING_WORLD")
CallToArms:RegisterEvent("LFG_UPDATE_RANDOM_INFO")
CallToArms:RegisterEvent("PLAYER_REGEN_ENABLED")
CallToArms:RegisterEvent("PLAYER_REGEN_DISABLED")
CallToArms:RegisterEvent("VARIABLES_LOADED")
CallToArms:RegisterEvent("CHAT_MSG_ADDON")
CallToArms:SetScript("OnEvent", OnEvent)

local Update = function(self)
	self.TankActive = false
	self.HealerActive = false
	self.DamageActive = false
	
	for i = 1, LFG_ROLE_NUM_SHORTAGE_TYPES do
		local Eligable, ForTank, ForHealer, ForDamage, ItemCount = GetLFGRoleShortageRewards(self.DungeonID, i)
		
		if (ItemCount and ItemCount > 0) then
			if ForTank then
				self.TankActive = true
			end
			
			if ForHealer then
				self.HealerActive = true
			end
			
			if ForDamage then
				self.DamageActive = true
			end
		end
	end
	
	if self.TankActive then
		if self.AnnounceTank then
			self[1]:Show()
			self:Sort()
			
			if Options.PrintChat then
				print(format("Call To Arms (%s): Tank bonus is active!", self.DungeonName))
			end
			
			self.AnnounceTank = false
		end
	else
		if (not self.AnnounceTank) then
			self[1]:Hide()
			self:Sort()
			
			if Options.AnnounceEndings then
				if Options.PrintChat then
					print(format("Call To Arms (%s): Tank bonus has ended.", self.DungeonName))
				end
			end
			
			self.AnnounceTank = true
		end
	end
	
	if self.HealerActive then
		if self.AnnounceHealer then
			self[2]:Show()
			self:Sort()
			
			if Options.PrintChat then
				print(format("Call To Arms (%s): Healer bonus is active!", self.DungeonName))
			end
			
			self.AnnounceHealer = false
		end
	else
		if (not self.AnnounceHealer) then
			self[2]:Hide()
			self:Sort()
			
			if Options.AnnounceEndings then
				if Options.PrintChat then
					print(format("Call To Arms (%s): Healer bonus has ended.", self.DungeonName))
				end
			end
			
			self.AnnounceHealer = true
		end	
	end
	
	if self.DamageActive then
		if self.AnnounceDPS then
			self[3]:Show()
			self:Sort()
			
			if Options.PrintChat then
				print(format("Call To Arms (%s): DPS bonus is active!", self.DungeonName))
			end
			
			self.AnnounceDPS = false
		end
	else
		if (not self.AnnounceDPS) then
			self[3]:Hide()
			self:Sort()
			
			if Options.AnnounceEndings then
				if Options.PrintChat then
					print(format("Call To Arms (%s): DPS bonus has ended.", self.DungeonName))
				end
			end
			
			self.AnnounceDPS = true
		end
	end
end

local OnFinished = function(self) -- Loop 3 times
	local Parent = self.Parent
	
	Parent.Count = Parent.Count - 1
	
	if (Parent.Count == 0) then
		self.Group:SetLooping(false)
		Parent.Count = 3
	end
end

local OnShow = function(self)
	self.FlashAnim:Play()
end

function CallToArms:NewModule(name, id)
	local Header = CreateFrame("Frame", nil, CallToArms)
	Header:SetSize(130, 18)
	Header:SetPoint("LEFT", UIParent, 8, 0)
	Header.Ela = 0
	Header:Hide()
	Header.DungeonID = id
	Header.DungeonName = name
	Header.Update = Update
	
	Header.Backdrop = Header:CreateTexture(nil, "BORDER")
	Header.Backdrop:SetPoint("TOPLEFT", Header, -1, 1)
	Header.Backdrop:SetPoint("BOTTOMRIGHT", Header, 1, -1)
	Header.Backdrop:SetTexture(BlankTex)
	Header.Backdrop:SetVertexColor(0, 0, 0)
	
	Header.Texture = Header:CreateTexture(nil, "ARTWORK")
	Header.Texture:SetPoint("TOPLEFT", Header, 0, 0)
	Header.Texture:SetPoint("BOTTOMRIGHT", Header, 0, 0)
	Header.Texture:SetTexture(BlankTex)
	Header.Texture:SetVertexColor(0.2, 0.2, 0.2)
	
	Header.Text = Header:CreateFontString(nil, "OVERLAY")
	Header.Text:SetPoint("LEFT", Header, 3, -0.5)
	Header.Text:SetFont(Font, 12)
	Header.Text:SetJustifyH("LEFT")
	Header.Text:SetShadowColor(0, 0, 0)
	Header.Text:SetShadowOffset(1.25, -1.25)
	Header.Text:SetText(name)
	
	Header.TankActive = false
	Header.HealerActive = false
	Header.DamageActive = false
	Header.AnnounceTank = true
	Header.AnnounceHealer = true
	Header.AnnounceDPS = true
	
	Header.Sort = function(self)
		self.LastShown = self
		local NumShown = 0
		
		for i = 1, 3 do
			self[i]:ClearAllPoints()
			
			if self[i]:IsShown() then
				self[i]:SetPoint("TOPLEFT", self.LastShown, "BOTTOMLEFT", 0, -1)
				
				self.LastShown = self[i]
				
				NumShown = NumShown + 1
			end
		end
		
		self.NumShown = NumShown
		
		if (NumShown > 0) then
			self:Show()
		else
			self:Hide()
		end
		
		CallToArms:SortHeaders()
	end
	
	for i = 1, 3 do
		local Bar = CreateFrame("Frame", nil, CallToArms)
		Bar:SetSize(130, 18)
		Bar:Hide()
		Bar.Ela = 0
		Bar.Count = 3
		
		Bar.Backdrop = Bar:CreateTexture(nil, "BORDER")
		Bar.Backdrop:SetPoint("TOPLEFT", Bar, -1, 1)
		Bar.Backdrop:SetPoint("BOTTOMRIGHT", Bar, 1, -1)
		Bar.Backdrop:SetTexture(BlankTex)
		Bar.Backdrop:SetVertexColor(0, 0, 0)
		
		Bar.Texture = Bar:CreateTexture(nil, "ARTWORK")
		Bar.Texture:SetPoint("TOPLEFT", Bar, 0, 0)
		Bar.Texture:SetPoint("BOTTOMRIGHT", Bar, 0, 0)
		Bar.Texture:SetTexture(BlankTex)
		Bar.Texture:SetVertexColor(0.3, 0.3, 0.3)
		
		Bar.Flash = Bar:CreateTexture(nil, "OVERLAY")
		Bar.Flash:SetPoint("TOPLEFT", Bar, 0, 0)
		Bar.Flash:SetPoint("BOTTOMRIGHT", Bar, 0, 0)
		Bar.Flash:SetTexture(BarTex)
		Bar.Flash:SetVertexColor(0.8, 0.8, 0.8)
		Bar.Flash:SetAlpha(1)
		Bar.Flash.Count = 3
		
		Bar.FlashAnim = CreateAnimationGroup(Bar.Flash)
		Bar.FlashAnim:SetLooping(true)
		
		Bar.FlashAnim.In = Bar.FlashAnim:CreateAnimation("Fade")
		Bar.FlashAnim.In:SetChange(1)
		Bar.FlashAnim.In:SetDuration(0.4)
		Bar.FlashAnim.In:SetSmoothing("inout")
		
		Bar.FlashAnim.Out = Bar.FlashAnim:CreateAnimation("Fade")
		Bar.FlashAnim.Out:SetChange(-1)
		Bar.FlashAnim.Out:SetDuration(0.4)
		Bar.FlashAnim.Out:SetOrder(2)
		Bar.FlashAnim.Out:SetSmoothing("inout")
		Bar.FlashAnim.Out:SetScript("OnFinished", OnFinished)
		
		Bar.Text = Bar:CreateFontString(nil, "OVERLAY")
		Bar.Text:SetPoint("LEFT", Bar, 3, -0.5)
		Bar.Text:SetFont(Font, 12)
		Bar.Text:SetJustifyH("LEFT")
		Bar.Text:SetShadowColor(0, 0, 0)
		Bar.Text:SetShadowOffset(1.25, -1.25)
		Bar.Text:SetText(RolesName[i])
		Bar.Text:SetTextColor(0.2, 0.8, 0.2)
		
		Bar:SetScript("OnShow", OnShow)
		
		if (i == 1) then
			Bar:SetPoint("TOPLEFT", Header, "BOTTOMLEFT", 0, -1)
		else
			Bar:SetPoint("TOP", Header[i-1], "BOTTOM", 0, -1)
		end
		
		Header[i] = Bar
	end
	
	self.Headers[name] = Header
end

local CreateMilitantCTAConfig = function()
	local Config = CreateFrame("Frame", "MilitantCallToArmsConfig", UIParent)
	Config:SetSize(200, 18)
	Config:SetPoint("CENTER", UIParent, 0, 0)
	Config:SetMovable(true)
	Config:EnableMouse(true)
	Config:SetUserPlaced(true)
	Config:RegisterForDrag("LeftButton")
	Config:SetScript("OnDragStart", Config.StartMoving)
	Config:SetScript("OnDragStop", Config.StopMovingOrSizing)
	
	Config.Backdrop = Config:CreateTexture(nil, "BORDER")
	Config.Backdrop:SetPoint("TOPLEFT", Config, -1, 1)
	Config.Backdrop:SetPoint("BOTTOMRIGHT", Config, 1, -1)
	Config.Backdrop:SetTexture(BlankTex)
	Config.Backdrop:SetVertexColor(0, 0, 0)
	
	Config.Texture = Config:CreateTexture(nil, "OVERLAY")
	Config.Texture:SetPoint("TOPLEFT", Config, 0, 0)
	Config.Texture:SetPoint("BOTTOMRIGHT", Config, 0, 0)
	Config.Texture:SetTexture(BarTex)
	Config.Texture:SetVertexColor(0.2, 0.2, 0.2)
	
	Config.Text = Config:CreateFontString(nil, "OVERLAY")
	Config.Text:SetPoint("LEFT", Config, 3, -0.5)
	Config.Text:SetFont(Font, 12)
	Config.Text:SetJustifyH("LEFT")
	Config.Text:SetShadowColor(0, 0, 0)
	Config.Text:SetShadowOffset(1.25, -1.25)
	Config.Text:SetText(format("|cff%02x%02x%02x", 0.90 * 255, 0.05 * 255, 0.07 * 255) .. "Militant|r " .. BATTLEGROUND_HOLIDAY)
	
	Config.CloseButton = CreateFrame("Frame", nil, Config)
	Config.CloseButton:SetPoint("TOPRIGHT", Config, 0, 0)
	Config.CloseButton:SetSize(18, 18)
	Config.CloseButton:SetScript("OnEnter", function(self) self.Label:SetTextColor(1, 0, 0) end)
	Config.CloseButton:SetScript("OnLeave", function(self) self.Label:SetTextColor(1, 1, 1) end)
	Config.CloseButton:SetScript("OnMouseUp", function() Config:Hide() end)
	
	Config.CloseButton.Label = Config.CloseButton:CreateFontString(nil, "OVERLAY")
	Config.CloseButton.Label:SetPoint("CENTER", Config.CloseButton, 0.5, -0.5)
	Config.CloseButton.Label:SetFont(Font, 18)
	Config.CloseButton.Label:SetText("×")
	Config.CloseButton.Label:SetShadowColor(0, 0, 0)
	Config.CloseButton.Label:SetShadowOffset(1.25, -1.25)
	
	local ConfigWindow = CreateFrame("Frame", nil, Config)
	ConfigWindow:SetSize(200, 48)
	ConfigWindow:SetPoint("TOP", Config, "BOTTOM", 0, -1)
	
	ConfigWindow.Backdrop = ConfigWindow:CreateTexture(nil, "BORDER")
	ConfigWindow.Backdrop:SetPoint("TOPLEFT", ConfigWindow, -1, 1)
	ConfigWindow.Backdrop:SetPoint("BOTTOMRIGHT", ConfigWindow, 1, -1)
	ConfigWindow.Backdrop:SetTexture(BlankTex)
	ConfigWindow.Backdrop:SetVertexColor(0, 0, 0)
	
	ConfigWindow.Inside = ConfigWindow:CreateTexture(nil, "BORDER")
	ConfigWindow.Inside:SetAllPoints()
	ConfigWindow.Inside:SetTexture(BlankTex)
	ConfigWindow.Inside:SetVertexColor(0.2, 0.2, 0.2)
	
	ConfigWindow.ButtonParent = CreateFrame("Frame", nil, ConfigWindow)
	ConfigWindow.ButtonParent:SetSize(200, 48)
	ConfigWindow.ButtonParent:SetPoint("CENTER", ConfigWindow, 0, 0)
	ConfigWindow.ButtonParent:SetFrameLevel(ConfigWindow:GetFrameLevel() + 4)
	ConfigWindow.ButtonParent:SetFrameStrata("HIGH")
	
	ConfigWindow.Boxes = {}
	
	for i = 1, 2 do
		local Checkbox = CreateFrame("Frame", nil, ConfigWindow.ButtonParent)
		Checkbox:SetSize(20, 20)
		
		Checkbox.BG = Checkbox:CreateTexture(nil, "BORDER")
		Checkbox.BG:SetTexture(BlankTex)
		Checkbox.BG:SetVertexColor(0, 0, 0)
		Checkbox.BG:SetPoint("TOPLEFT", Checkbox, 0, 0)
		Checkbox.BG:SetPoint("BOTTOMRIGHT", Checkbox, 0, 0)
		
		Checkbox.Tex = Checkbox:CreateTexture(nil, "OVERLAY")
		Checkbox.Tex:SetTexture(BarTex)
		Checkbox.Tex:SetPoint("TOPLEFT", Checkbox, 1, -1)
		Checkbox.Tex:SetPoint("BOTTOMRIGHT", Checkbox, -1, 1)
		
		Checkbox.Text = Checkbox:CreateFontString(nil, "OVERLAY")
		Checkbox.Text:SetFont(Font, 12)
		Checkbox.Text:SetPoint("LEFT", Checkbox, "RIGHT", 3, 0)
		Checkbox.Text:SetJustifyH("LEFT")
		Checkbox.Text:SetShadowColor(0, 0, 0)
		Checkbox.Text:SetShadowOffset(1.25, -1.25)
		
		if (i == 1) then
			Checkbox:SetPoint("TOPLEFT", ConfigWindow.ButtonParent, 3, -3)
		else
			Checkbox:SetPoint("TOPLEFT", ConfigWindow.Boxes[i-1], "BOTTOMLEFT", 0, -2)
		end
		
		ConfigWindow.Boxes[i] = Checkbox
	end
	
	if Options.PrintChat then
		ConfigWindow.Boxes[1].Tex:SetVertexColor(0, 0.8, 0)
	else
		ConfigWindow.Boxes[1].Tex:SetVertexColor(0.8, 0, 0)
	end
	
	if Options.AnnounceEndings then
		ConfigWindow.Boxes[2].Tex:SetVertexColor(0, 0.8, 0)
	else
		ConfigWindow.Boxes[2].Tex:SetVertexColor(0.8, 0, 0)
	end
	
	ConfigWindow.Boxes[1].Text:SetText("Announce bonuses in chat window")
	ConfigWindow.Boxes[1]:SetScript("OnMouseUp", function(self)
		if Options.PrintChat then
			Militant_CallToArms.PrintChat = false
			Options.PrintChat = false
			
			self.Tex:SetVertexColor(0.8, 0, 0)
		else
			Militant_CallToArms.PrintChat = true
			Options.PrintChat = true
			
			self.Tex:SetVertexColor(0, 0.8, 0)
		end
	end)
	
	ConfigWindow.Boxes[2].Text:SetText("Announce when bonus queues end")
	ConfigWindow.Boxes[2]:SetScript("OnMouseUp", function(self)
		if Options.AnnounceEndings then
			Militant_CallToArms.AnnounceEndings = false
			Options.AnnounceEndings = false
			
			self.Tex:SetVertexColor(0.8, 0, 0)
		else
			Militant_CallToArms.AnnounceEndings = true
			Options.AnnounceEndings = true
			
			self.Tex:SetVertexColor(0, 0.8, 0)
		end
	end)
end

local SlashCommand = function(cmd)
	if (cmd == "toggle") then
		if CallToArms:IsShown() then
			CallToArms:Hide()
		else
			CallToArms:Show()
		end
	elseif (cmd == "config") then
		if (not MilitantCallToArmsConfig) then
			CreateMilitantCTAConfig()
			
			return
		end
		
		if MilitantCallToArmsConfig:IsShown() then
			MilitantCallToArmsConfig:Hide()
		else
			MilitantCallToArmsConfig:Show()
		end
	else
		if CallToArms:IsShown() then
			CallToArms:Hide()
		else
			CallToArms:Show()
		end
	end
end

SLASH_MILITANTCALLTOARMS1 = "/mcta"
SlashCmdList["MILITANTCALLTOARMS"] = SlashCommand